@keyframes snowfall {
    0% {
      transform: translateY(0);
      opacity: 1;
    }
    100% {
      transform: translateY(100vh);
      opacity: 0;
    }
  }
  
  .animate-snowfall {
    animation: snowfall linear infinite;
  }